/home/patel_zeel/venvs/skgpytorch/bin/python /home/patel_zeel/skgpytorch/zexperiments/test1.py boston &
/home/patel_zeel/venvs/skgpytorch/bin/python /home/patel_zeel/skgpytorch/zexperiments/test1.py autompg &
/home/patel_zeel/venvs/skgpytorch/bin/python /home/patel_zeel/skgpytorch/zexperiments/test1.py laptops &
